@@ -1,9 +1,15 @@
+require 'puppet/util/puppetdb'
+require 'puppet/util/puppetdb/command_names'
+
 Puppet::Face.define(:node, '0.0.1') do
+
+  CommandDeactivateNode = Puppet::Util::Puppetdb::CommandNames::CommandDeactivateNode
+
   action :deactivate do
     summary "Deactivate a set of nodes in PuppetDB"
     arguments "<node> [<node> ...]"
     description <<-DESC
-      This will issue 'deactivate node' commands to the PuppetDB server for
+      This will issue '#{CommandDeactivateNode}' commands to the PuppetDB server for
       each node specified. The server is found by looking in
       $confdir/puppetdb.conf. If any command submissions fail, the process will
     be aborted.
@@ -23,7 +29,7 @@ Puppet::Face.define(:node, '0.0.1') do
 
     when_rendering(:console) do |value|
       value.map do |node,uuid|
-        "Submitted 'deactivate node' for #{node} with UUID #{uuid}"
+        "Submitted '#{CommandDeactivateNode}' for #{node} with UUID #{uuid}"
       end.join("\n")
     end
   end
