@@ -4,11 +4,12 @@ require 'puppet/util/puppetdb'
 
 class Puppet::Resource::Catalog::Puppetdb < Puppet::Indirector::REST
   include Puppet::Util::Puppetdb
+  include Puppet::Util::Puppetdb::CommandNames
 
   def save(request)
     catalog = munge_catalog(request.instance)
 
-    submit_command(request, catalog, 'replace catalog', 1)
+    submit_command(request.key, catalog, CommandReplaceCatalog, 2)
   end
 
   def find(request)
@@ -29,6 +30,7 @@ class Puppet::Resource::Catalog::Puppetdb < Puppet::Indirector::REST
     sort_unordered_metaparams(data)
     munge_edges(data)
     synthesize_edges(data, catalog)
+    filter_keys(hash)
 
     hash
   end
@@ -223,6 +225,20 @@ class Puppet::Resource::Catalog::Puppetdb < Puppet::Indirector::REST
     hash
   end
 
+  def filter_keys(hash)
+    hash['metadata'].delete_if do |k,v|
+      k != 'api_version'
+    end
+
+    hash['data'].delete_if do |k,v|
+      ! ['name', 'version', 'edges', 'resources'].include?(k)
+    end
+
+    hash.delete_if do |k,v|
+      ! ['metadata', 'data'].include?(k)
+    end
+  end
+
   def resource_ref_to_hash(ref)
     ref =~ /^([^\[\]]+)\[(.+)\]$/m
     {'type' => $1, 'title' => $2}
