@@ -1,9 +1,12 @@
 #!/usr/bin/env ruby
 
 require 'spec_helper'
+require 'puppet'
 require 'puppet/face'
+require 'puppet/network/http_pool'
 
-describe Puppet::Face[:node, :current] do
+describe "node face: status" do
+  let(:subject) { Puppet::Face[:node, :current] }
   let(:headers) do
     {
       "Accept" => "application/json",
@@ -21,7 +24,7 @@ describe Puppet::Face[:node, :current] do
 
     nodes = %w[a b c d e]
     nodes.each do |node|
-      http.expects(:get).with("/status/nodes/#{node}", headers)
+      http.expects(:get).with("/v2/nodes/#{node}", headers)
     end
 
     subject.status(*nodes)
@@ -33,7 +36,7 @@ describe Puppet::Face[:node, :current] do
 
     node = "foo/+*&bar"
 
-    http.expects(:get).with("/status/nodes/foo%2F%2B%2A%26bar", headers)
+    http.expects(:get).with("/v2/nodes/foo%2F%2B%2A%26bar", headers)
 
     subject.status(node)
   end
