@@ -3,14 +3,17 @@
 require 'spec_helper'
 
 require 'puppet/indirector/catalog/puppetdb'
+require 'puppet/util/puppetdb/command_names'
 
 describe Puppet::Resource::Catalog::Puppetdb do
   before :each do
-    Puppet::Util::Puppetdb.stubs(:load_puppetdb_config).returns ['localhost', 0]
+    Puppet::Util::Puppetdb.stubs(:server).returns 'localhost'
+    Puppet::Util::Puppetdb.stubs(:port).returns 0
   end
 
   describe "#save" do
     let(:response) { Net::HTTPOK.new('1.1', 200, 'OK') }
+    let(:http)     { mock 'http' }
     let(:catalog) do
       cat = Puppet::Resource::Catalog.new('foo')
       cat.add_resource(Puppet::Resource.new(:file, 'my_file'))
@@ -19,6 +22,7 @@ describe Puppet::Resource::Catalog::Puppetdb do
 
     before :each do
       response.stubs(:body).returns '{"uuid": "a UUID"}'
+      Puppet::Network::HttpPool.expects(:http_instance).returns http
     end
 
     def save
@@ -26,14 +30,14 @@ describe Puppet::Resource::Catalog::Puppetdb do
     end
 
     it "should POST the catalog command as a URL-encoded PSON string" do
-      payload_str = subject.munge_catalog(catalog).to_pson
+      command_payload = subject.munge_catalog(catalog)
       payload = {
-        :command => "replace catalog",
-        :version => 1,
-        :payload => payload_str,
+        :command => Puppet::Util::Puppetdb::CommandNames::CommandReplaceCatalog,
+        :version => 2,
+        :payload => command_payload,
       }.to_pson
 
-      subject.expects(:http_post).with do |request,uri,body,headers|
+      http.expects(:post).with do |uri, body, headers|
         body =~ /payload=(.+)/
         @sent_payload = $1
       end.returns response
@@ -42,6 +46,18 @@ describe Puppet::Resource::Catalog::Puppetdb do
 
       CGI.unescape(@sent_payload).should == payload
     end
+
+    it "should log a deprecation warning if one is returned from PuppetDB" do
+      response['x-deprecation'] = 'A horrible deprecation warning!'
+
+      Puppet.expects(:deprecation_warning).with do |msg|
+        msg =~ /A horrible deprecation warning!/
+      end
+
+      http.stubs(:post).returns response
+
+      save
+    end
   end
 
   describe "catalog transformation methods" do
@@ -612,6 +628,14 @@ describe Puppet::Resource::Catalog::Puppetdb do
           [res['type'], res['title']].should_not == ['Notify', 'something']
         end
       end
+
+      it "should have the correct set of keys" do
+        result = subject.munge_catalog(catalog)
+
+        result.keys.should =~ ['metadata', 'data']
+        result['metadata'].keys.should =~ ['api_version']
+        result['data'].keys.should =~ ['name', 'version', 'edges', 'resources']
+      end
     end
   end
 end
