@@ -2,20 +2,25 @@
 require 'spec_helper'
 
 require 'puppet/indirector/facts/puppetdb'
+require 'puppet/util/puppetdb/command_names'
 
 describe Puppet::Node::Facts::Puppetdb do
+
+  CommandReplaceFacts = Puppet::Util::Puppetdb::CommandNames::CommandReplaceFacts
+
   before :each do
-    Puppet::Util::Puppetdb.stubs(:load_puppetdb_config).returns ['localhost', 0]
+    Puppet::Util::Puppetdb.stubs(:server).returns 'localhost'
+    Puppet::Util::Puppetdb.stubs(:port).returns 0
     Puppet::Node::Facts.indirection.stubs(:terminus).returns(subject)
   end
 
   describe "#save" do
     let(:response) { Net::HTTPOK.new('1.1', 200, 'OK') }
-    let(:facts) do
-      Puppet::Node::Facts.new('foo')
-    end
+    let(:facts)    { Puppet::Node::Facts.new('foo') }
+    let(:http)     { mock 'http' }
 
     before :each do
+      Puppet::Network::HttpPool.expects(:http_instance).returns http
       response.stubs(:body).returns '{"uuid": "a UUID"}'
     end
 
@@ -25,12 +30,12 @@ describe Puppet::Node::Facts::Puppetdb do
 
     it "should POST the facts command as a URL-encoded PSON string" do
       payload = {
-        :command => "replace facts",
+        :command => CommandReplaceFacts,
         :version => 1,
         :payload => facts.to_pson,
       }.to_pson
 
-      subject.expects(:http_post).with do |request,uri,body,headers|
+      http.expects(:post).with do |uri, body, headers|
         body =~ /payload=(.+)/
         @sent_payload = $1
       end.returns response
@@ -43,13 +48,7 @@ describe Puppet::Node::Facts::Puppetdb do
     it "should stringify fact values before submitting" do
       facts.values['something'] = 100
 
-      payload = {
-        :command => "replace facts",
-        :version => 1,
-        :payload => facts.to_pson,
-      }.to_pson
-
-      subject.expects(:http_post).with do |request,uri,body,headers|
+      http.expects(:post).with do |uri, body, headers|
         body =~ /payload=(.+)/
         @sent_payload = $1
       end.returns response
@@ -67,14 +66,12 @@ describe Puppet::Node::Facts::Puppetdb do
 
   describe "#find" do
     def find_facts()
-      Puppet::Node::Facts.indirection.find('facts')
+      Puppet::Node::Facts.indirection.find('some_node')
     end
 
     it "should return the facts if they're found" do
-      facts = {'a' => '1',
-                'b' => '2'}
-      body = {:name => 'some_node',
-              :facts => facts}.to_pson
+      body = [{:node => 'some_node', :name => 'a', :value => '1'},
+              {:node => 'some_node', :name => 'b', :value => '2'}].to_pson
 
       response = Net::HTTPOK.new('1.1', 200, 'OK')
       response.stubs(:body).returns body
@@ -84,7 +81,7 @@ describe Puppet::Node::Facts::Puppetdb do
       result = find_facts
       result.should be_a(Puppet::Node::Facts)
       result.name.should == 'some_node'
-      result.values.should include(facts)
+      result.values.should include('a' => '1', 'b' => '2')
     end
 
     it "should return nil if no facts are found" do
@@ -113,6 +110,23 @@ describe Puppet::Node::Facts::Puppetdb do
         find_facts
       }.to raise_error Puppet::Error, /Everything is terrible!/
     end
+
+    it "should log a deprecation warning if one is returned from PuppetDB" do
+      response = Net::HTTPOK.new('1.1', 200, 'OK')
+      response['x-deprecation'] = "This is deprecated!"
+
+      body = [].to_pson
+
+      response.stubs(:body).returns body
+
+      subject.stubs(:http_get).returns response
+
+      Puppet.expects(:deprecation_warning).with do |msg|
+        msg =~ /This is deprecated!/
+      end
+
+      find_facts
+    end
   end
 
   describe "#search" do
@@ -127,6 +141,9 @@ describe Puppet::Node::Facts::Puppetdb do
       }
 
       response.stubs(:body).returns '["foo", "bar", "baz"]'
+      response.stubs(:body).returns '[{"name": "foo", "deactivated": null, "catalog_timestamp": null, "facts_timestamp": null, "report_timestamp": null},
+                                      {"name": "bar", "deactivated": null, "catalog_timestamp": null, "facts_timestamp": null, "report_timestamp": null},
+                                      {"name": "baz", "deactivated": null, "catalog_timestamp": null, "facts_timestamp": null, "report_timestamp": null}]'
       subject.stubs(:http_get).returns response
 
       search_facts(args).should == ['foo', 'bar', 'baz']
@@ -143,37 +160,19 @@ describe Puppet::Node::Facts::Puppetdb do
       end.to raise_error(Puppet::Error, /Fact search against keys of type 'wrong' is unsupported/)
     end
 
-    it "should add a filter against only active nodes" do
-      args = {
-        'facts.kernel.eq' => 'Linux',
-      }
-
-      query = CGI.escape(["and", ["=", ["node", "active"], true],
-                                 ["=", ["fact", "kernel"], "Linux"]].to_pson)
-
-      response.stubs(:body).returns '[]'
-
-      subject.expects(:http_get).with do |_,url,_|
-        url.should == "/nodes?query=#{query}"
-      end.returns response
-
-      search_facts(args)
-    end
-
     it "should combine multiple terms with 'and'" do
       args = {
         'facts.kernel.eq' => 'Linux',
         'facts.uptime.eq' => '10 days',
       }
 
-      query = CGI.escape(["and", ["=", ["node", "active"], true],
-                                 ["=", ["fact", "kernel"], "Linux"],
+      query = CGI.escape(["and", ["=", ["fact", "kernel"], "Linux"],
                                  ["=", ["fact", "uptime"], "10 days"]].to_pson)
 
       response.stubs(:body).returns '[]'
 
       subject.expects(:http_get).with do |_,url,_|
-        url.should == "/nodes?query=#{query}"
+        url.should == "/v2/nodes?query=#{query}"
       end.returns response
 
       search_facts(args)
@@ -184,13 +183,12 @@ describe Puppet::Node::Facts::Puppetdb do
         'facts.kernel.ne' => 'Linux',
       }
 
-      query = CGI.escape(["and", ["=", ["node", "active"], true],
-                                 ["not", ["=", ["fact", "kernel"], "Linux"]]].to_pson)
+      query = CGI.escape(["and", ["not", ["=", ["fact", "kernel"], "Linux"]]].to_pson)
 
       response.stubs(:body).returns '[]'
 
       subject.expects(:http_get).with do |_,url,_|
-        url.should == "/nodes?query=#{query}"
+        url.should == "/v2/nodes?query=#{query}"
       end.returns response
 
       search_facts(args)
@@ -201,13 +199,12 @@ describe Puppet::Node::Facts::Puppetdb do
         'facts.kernel' => 'Linux',
       }
 
-      query = CGI.escape(["and", ["=", ["node", "active"], true],
-                                 ["=", ["fact", "kernel"], "Linux"]].to_pson)
+      query = CGI.escape(["and", ["=", ["fact", "kernel"], "Linux"]].to_pson)
 
       response.stubs(:body).returns '[]'
 
       subject.expects(:http_get).with do |_,url,_|
-        url.should == "/nodes?query=#{query}"
+        url.should == "/v2/nodes?query=#{query}"
       end.returns response
 
       search_facts(args)
@@ -224,13 +221,12 @@ describe Puppet::Node::Facts::Puppetdb do
           "facts.kernel.#{name}" => 'Linux',
         }
 
-        query = CGI.escape(["and", ["=", ["node", "active"], true],
-                                   [operator, ["fact", "kernel"], "Linux"]].to_pson)
+        query = CGI.escape(["and", [operator, ["fact", "kernel"], "Linux"]].to_pson)
 
         response.stubs(:body).returns '[]'
 
         subject.expects(:http_get).with do |_,url,_|
-          url.should == "/nodes?query=#{query}"
+          url.should == "/v2/nodes?query=#{query}"
         end.returns response
 
         search_facts(args)
@@ -247,5 +243,18 @@ describe Puppet::Node::Facts::Puppetdb do
         search_facts(nil)
       end.to raise_error(Puppet::Error, /Could not perform inventory search from PuppetDB at localhost:0: \[400 Bad Request\] Something bad happened!/)
     end
+
+    it "should log a deprecation warning if one is returned from PuppetDB" do
+      response['x-deprecation'] = "This is deprecated!"
+      response.stubs(:body).returns '[]'
+
+      subject.stubs(:http_get).returns response
+
+      Puppet.expects(:deprecation_warning).with do |msg|
+        msg =~ /This is deprecated!/
+      end
+
+      search_facts(nil)
+    end
   end
 end
