@@ -4,10 +4,49 @@
   (:import (com.jolbox.bonecp BoneCPDataSource BoneCPConfig)
            (java.util.concurrent TimeUnit))
   (:require [clojure.java.jdbc :as sql]
+            [clojure.string :as string]
             [clojure.tools.logging :as log]
             [com.puppetlabs.utils :as utils])
   (:use com.puppetlabs.jdbc.internal))
 
+
+(defn valid-jdbc-query?
+  "Most SQL queries generated in the PuppetDB code base are represented internally
+  as a vector whose first item is the SQL string (with optional '?' placeholders),
+  and whose remaining items (if any) are simple data types that can be passed
+  to a JDBC prepared statement as parameter values to bind to the placeholders
+  in the SQL string.  This function validates that a form complies to this structure.
+  It is intended primarily for use in pre- and post-conditions, for validation."
+  [q]
+  (and
+    (vector? q)
+    (string? (first q))
+    (every? (complement coll?) (rest q))))
+
+;; ## String operations
+
+(defn dashes->underscores
+  "Accepts a string or a keyword as an argument, replaces all occurrences of the
+  dash/hyphen character with an underscore, and returns the same type (string
+  or keyword) that was passed in.  This is useful for translating data structures
+  from their wire format to the format that is needed for JDBC."
+  [str]
+  (let [result (string/replace (name str) \- \_)]
+    (if (keyword? str)
+      (keyword result)
+      result)))
+
+(defn underscores->dashes
+  "Accepts a string or a keyword as an argument, replaces all occurrences of the
+  underscore character with a dash, and returns the same type (string
+  or keyword) that was passed in.  This is useful for translating data structures
+  from their JDBC-compatible representation to their wire format representation."
+  [str]
+  (let [result (string/replace (name str) \_ \-)]
+    (if (keyword? str)
+      (keyword result)
+      result)))
+
 (defn convert-result-arrays
   "Converts Java and JDBC arrays in a result set using the provided function
   (eg. vec, set). Values which aren't arrays are unchanged."
@@ -105,15 +144,18 @@
   "Create a new database connection pool"
   [{:keys [classname subprotocol subname username password
            partition-conn-min partition-conn-max partition-count
-           stats log-statements log-slow-statements]
+           stats log-statements log-slow-statements
+           conn-max-age conn-keep-alive]
     :or   {partition-conn-min  1
-           partition-conn-max  10
-           partition-count     5
+           partition-conn-max  50
+           partition-count     1
            stats               true
            ;; setting this to a String value, because that's what it would
            ;;  be in the config file and we're manually converting it to a boolean
            log-statements      "true"
-           log-slow-statements 10}
+           log-slow-statements 10
+           conn-max-age        60
+           conn-keep-alive     240}
     :as   db}]
   ;; Load the database driver class
   (Class/forName classname)
@@ -125,6 +167,8 @@
                           (.setMaxConnectionsPerPartition partition-conn-max)
                           (.setPartitionCount partition-count)
                           (.setStatisticsEnabled stats)
+                          (.setIdleMaxAgeInMinutes conn-max-age)
+                          (.setIdleConnectionTestPeriodInMinutes conn-keep-alive)
                           ;; paste the URL back together from parts.
                           (.setJdbcUrl (str "jdbc:" subprotocol ":" subname))
                           (.setConnectionHook (connection-hook log-statements? log-slow-statements)))]
