@@ -51,17 +51,17 @@
             [com.puppetlabs.jetty :as jetty]
             [com.puppetlabs.mq :as mq]
             [com.puppetlabs.utils :as pl-utils]
-            [clj-time.core :as time]
             [clojure.java.jdbc :as sql]
             [clojure.tools.logging :as log]
-            [clojure.tools.nrepl.server :as nrepl]
-            [swank.swank :as swank]
+            [cheshire.core :as json]
             [com.puppetlabs.puppetdb.http.server :as server])
   (:use [clojure.java.io :only [file]]
-        [clojure.tools.nrepl.transport :only (tty tty-greeting)]
+        [clj-time.core :only [ago secs days]]
         [clojure.core.incubator :only (-?>)]
+        [com.puppetlabs.time :only [to-secs parse-period format-period]]
         [com.puppetlabs.jdbc :only (with-transacted-connection)]
         [com.puppetlabs.utils :only (cli! configure-logging! inis-to-map with-error-delivery)]
+        [com.puppetlabs.repl :only (start-repl)]
         [com.puppetlabs.puppetdb.scf.migrate :only [migrate!]]
         [com.puppetlabs.puppetdb.version :only [version update-info]]))
 
@@ -98,9 +98,14 @@
   This function doesn't terminate. If we encounter an exception during
   compaction, the operation will be retried after `interval` millis.
 
-  This function will perform database garbage collection, and will also
-  deactivate nodes more stale than `node-ttl-days`"
-  [db interval node-ttl-days]
+  This function will perform database garbage collection.  This entails
+  cleaning out catalogs and facts that are no longer in use, deactivating nodes
+  that haven't been active in the last `node-ttl-seconds`, and clearing out
+  reports that are older than `report-ttl-seconds`"
+  [db interval node-ttl-seconds report-ttl-seconds]
+  {:pre [(pos? interval)
+         (>= node-ttl-seconds 0)
+         (>= report-ttl-seconds 0)]}
   (let [sleep #(Thread/sleep (* 60 1000 interval))]
     (pl-utils/keep-going
      (fn [exception]
@@ -112,12 +117,20 @@
       (with-transacted-connection db
         (scf-store/garbage-collect!)))
 
-     (when (pos? node-ttl-days)
+     (when (pos? node-ttl-seconds)
        (pl-utils/demarcate
-        (format "sweep of stale nodes (%s day threshold)" node-ttl-days)
+        (format "sweep of stale nodes (threshold: %s)"
+          (format-period (secs node-ttl-seconds)))
         (with-transacted-connection db
-          (doseq [node (scf-store/stale-nodes (time/ago (time/days node-ttl-days)))]
-            (send-command! "deactivate node" 1 node)))))
+          (doseq [node (scf-store/stale-nodes (ago (secs node-ttl-seconds)))]
+            (send-command! "deactivate node" 1 (json/generate-string node))))))
+
+     (when (pos? report-ttl-seconds)
+       (pl-utils/demarcate
+        (format "sweep of stale reports (threshold: %s)"
+          (format-period (secs report-ttl-seconds)))
+        (with-transacted-connection db
+          (scf-store/delete-reports-older-than! (ago (secs report-ttl-seconds))))))
 
      (sleep))))
 
@@ -128,7 +141,7 @@
   (let [{:keys [version newer link]} (try
                                        (update-info update-server db)
                                        (catch Throwable e
-                                         (log/debug e "Could not retrieve update information")))
+                                         (log/debug e (format "Could not retrieve update information (%s)" update-server))))
         link-str                     (if link
                                        (format "Visit %s for details." link)
                                        "")
@@ -160,6 +173,54 @@
    :post [(map? config)]}
   (assoc-in config [:jetty :client-auth] :need))
 
+(defn configure-node-ttl
+  "Helper function that parses the `node-ttl` setting from the config file
+  (if present) into our internal representation (in seconds).  Also supports
+  `node-ttl-days` for backward compatibility with previous versions."
+  [database]
+  {:pre  [(map? database)]
+   :post [(map? %)
+          (not (contains? % :node-ttl))
+          (not (contains? % :node-ttl-days))]}
+  (cond
+    (:node-ttl database)      (-> database
+                                  (assoc :node-ttl-seconds
+                                    (to-secs (parse-period (:node-ttl database))))
+                                  (dissoc :node-ttl)
+                                  (dissoc :node-ttl-days))
+    (:node-ttl-days database) (-> database
+                                  (assoc :node-ttl-seconds
+                                    (to-secs (days (:node-ttl-days database))))
+                                  (dissoc :node-ttl-days))
+    :else                     database))
+
+(defn configure-report-ttl
+  "Helper function that parses the `report-ttl` setting from the config file
+  (if present) into our internal representation (in seconds)"
+  [database]
+  {:pre  [(map? database)]
+   :post [(map? %)
+          (not (contains? % :report-ttl))]}
+  (if-let [report-ttl (:report-ttl database)]
+    (-> database
+      (assoc :report-ttl-seconds (to-secs (parse-period report-ttl)))
+      (dissoc :report-ttl))
+    database))
+
+(defn configure-database-ttls
+  "Helper function that munges the supported permutations of our `ttl` settings
+  (if present) from their config file representation to our internal
+  representation (in seconds)"
+  [database]
+  {:pre  [(map? database)]
+   :post [(map? database)
+          (not (contains? % :report-ttl))
+          (not (contains? % :node-ttl))
+          (not (contains? % :node-ttl-days))]}
+  (-> database
+      configure-report-ttl
+      configure-node-ttl))
+
 (defn configure-database
   "Update the supplied config map with information about the database. Adds a
   default hsqldb and a gc-interval of 60 minutes. If a single part of the
@@ -172,11 +233,11 @@
         default-db   {:classname   "org.hsqldb.jdbcDriver"
                       :subprotocol "hsqldb"
                       :subname     (format "file:%s;hsqldb.tx=mvcc;sql.syntax_pgs=true" (file vardir "db"))}
-        default-opts {:gc-interval   60
-                      :node-ttl-days 0}
-        db           (merge default-opts
-                            (or database default-db))]
-    (assoc config :database db)))
+        default-opts {:gc-interval        60
+                      :node-ttl-seconds   0
+                      :report-ttl-seconds (to-secs (days 7))}
+        db           (configure-database-ttls (or database default-db))]
+    (assoc config :database (merge default-opts db))))
 
 (defn validate-vardir
   "Checks that `vardir` is specified, exists, and is writeable, throwing
@@ -246,13 +307,15 @@
   [& args]
   (let [[options _]                                (cli! args)
         initial-config                             {:debug (:debug options)}
-        {:keys [jetty database global] :as config} (parse-config! (:config options) initial-config)
+        {:keys [jetty database global command-processing]
+            :as config}                            (parse-config! (:config options) initial-config)
         vardir                                     (validate-vardir (:vardir global))
         update-server                              (:update-server global "http://updates.puppetlabs.com/check-for-updates")
         resource-query-limit                       (get global :resource-query-limit 20000)
         db                                         (pl-jdbc/pooled-datasource database)
-        db-gc-minutes                              (get database :gc-interval 60)
-        node-ttl-days                              (get database :node-ttl-days)
+        db-gc-minutes                              (get database :gc-interval)
+        node-ttl-seconds                           (get database :node-ttl-seconds)
+        report-ttl-seconds                         (get database :report-ttl-seconds)
         mq-dir                                     (str (file vardir "mq"))
         discard-dir                                (file mq-dir "discarded")
         globals                                    {:scf-db               db
@@ -269,8 +332,12 @@
     ;; Add a shutdown hook where we can handle any required cleanup
     (pl-utils/add-shutdown-hook! on-shutdown)
 
-    ;; Ensure the database is migrated to the latest version
+    ;; Ensure the database is migrated to the latest version, and warn if it's
+    ;; deprecated. We do this in a single connection because HSQLDB seems to
+    ;; get confused if the database doesn't exist but we open and close a
+    ;; connection without creating anything.
     (sql/with-connection db
+      (scf-store/warn-on-db-deprecation!)
       (migrate!))
 
     ;; Initialize database-dependent metrics
@@ -279,8 +346,10 @@
     (let [error         (promise)
           broker        (do
                           (log/info "Starting broker")
-                          (mq/start-broker! (mq/build-embedded-broker mq-dir)))
-          command-procs (let [nthreads (get-in config [:command-processing :threads])]
+                          (mq/start-broker!
+                            (mq/build-embedded-broker
+                              "localhost" mq-dir command-processing)))
+          command-procs (let [nthreads (command-processing :threads)]
                           (log/info (format "Starting %d command processor threads" nthreads))
                           (vec (for [n (range nthreads)]
                                  (future (with-error-delivery error
@@ -296,18 +365,13 @@
           db-gc         (do
                           (log/info (format "Starting database sweeper (%d minute interval)" db-gc-minutes))
                           (future (with-error-delivery error
-                                    (sweep-database! db db-gc-minutes node-ttl-days))))]
+                                    (sweep-database! db db-gc-minutes node-ttl-seconds report-ttl-seconds))))]
 
       ;; Start debug REPL if necessary
       (let [{:keys [enabled type host port] :or {type "nrepl" host "localhost"}} (:repl config)]
         (when (= "true" enabled)
           (log/warn (format "Starting %s server on port %d" type port))
-          (cond
-           (= type "nrepl")
-           (nrepl/start-server :port port :transport-fn tty :greeting-fn tty-greeting)
-
-           (= type "swank")
-           (swank/start-server :host host :port port))))
+          (start-repl type host port)))
 
       (let [exception (deref error)]
         (doseq [cp command-procs]
