@@ -9,19 +9,82 @@
 ;; the migration function will be invoked, and the schema version and current
 ;; time will be recorded in the schema_migrations table.
 ;;
+;; NOTE: in order to support bug-fix schema changes to older branches without
+;; breaking the ability to upgrade, it is possible to define a sequence of
+;; migrations with non-sequential integers.  e.g., if the 1.0.x branch
+;; contains migrations 1-5, and the 2.0.x branch contains schema migrations
+;; 1-10, and then a bugfix schema change (such as creating or adding an index)
+;; is identified, this migration can be defined as #11 in both branches.  Code
+;; in the 1.0.x branch should happily apply #11 even though it does not have
+;; a definition for 6-10.  Then when a 1.0.x user upgrades to 2.0.x, migrations
+;; 6-10 will be applied, and 11 will be skipped because it's already been run.
+;; Because of this, it is crucial to be extremely careful about numbering new
+;; migrations if they are going into multiple branches.  It's also crucial to
+;; be absolutely certain that the schema change in question is compatible
+;; with both branches and that the migrations missing from the earlier branch
+;; can reasonably and safely be applied *after* the bugfix migration, because
+;; that is what will happen for upgrading users.
+;;
+;; In short, here are some guidelines re: applying schema changes to multiple
+;; branches:
+;;
+;; 1. If at all possible, avoid it.
+;; 2. Seriously, are you sure you need to do this? :)
+;; 3. OK, if you really must do it, make sure that the schema change in question
+;;    is as independent as humanly possible.  For example, things like creating
+;;    or dropping an index on a table should be fairly self-contained.  You should
+;;    think long and hard about any change more complex than that.
+;; 4. Determine what the latest version of the schema is in each of the two branches.
+;; 5. Examine every migration that exists in the newer branch but not the older
+;;    branch, and make sure that your new schema change will not conflict with
+;;    *any* of those migrations.  Your change must be able to execute successfully
+;;    regardless of whether it is applied BEFORE all of those migrations or AFTER
+;;    them.
+;; 6. If you're certain you've met the conditions described above, choose the next
+;;    available integer from the *newer* branch and add your migration to both
+;;    branches using this integer.  This will result in a gap between the integers
+;;    in the migrations array in the old branch, but that is not a problem.
+;;
 ;; _TODO: consider using multimethods for migration funcs_
 
 (ns com.puppetlabs.puppetdb.scf.migrate
   (:require [clojure.java.jdbc :as sql]
             [clojure.tools.logging :as log]
-            [clojure.string :as string])
-  (:use [clj-time.coerce :only [to-timestamp]]
+            [clojure.string :as string]
+            [com.puppetlabs.utils :as utils])
+  (:use [clojure.set]
+        [clj-time.coerce :only [to-timestamp]]
         [clj-time.core :only [now]]
         [com.puppetlabs.jdbc :only [query-to-vec]]
         [com.puppetlabs.puppetdb.scf.storage :only [sql-array-type-string
                                                     sql-current-connection-database-name
                                                     sql-current-connection-database-version]]))
 
+(defn- drop-constraints
+  "Drop all constraints of given `constraint-type` on `table`."
+  [table constraint-type]
+  (let [results     (query-to-vec
+                      (str "SELECT constraint_name FROM information_schema.table_constraints "
+                           "WHERE LOWER(table_name) = LOWER(?) AND LOWER(constraint_type) = LOWER(?)")
+                      table constraint-type)
+        constraints (map :constraint_name results)]
+    (if (seq constraints)
+      (apply sql/do-commands
+             (for [constraint constraints]
+               (format "ALTER TABLE %s DROP CONSTRAINT %s" table constraint)))
+      (throw (IllegalArgumentException. (format "No %s constraint exists on the table '%s'" constraint-type table))))))
+
+(defn- drop-primary-key
+  "Drop the primary key on the given `table`."
+  [table]
+  (drop-constraints table "primary key"))
+
+(defn- drop-foreign-keys
+  "Drop all foreign keys on the given `table`. Does not currently support
+  selecting a single key to drop."
+  [table]
+  (drop-constraints table "foreign key"))
+
 (defn initialize-store
   "Create the initial database schema."
   []
@@ -134,12 +197,7 @@
    [(to-timestamp (now))])
 
   ;; First we get rid of the existing foreign key to certnames
-  (let [[result & _] (query-to-vec
-                      (str "SELECT constraint_name FROM information_schema.table_constraints "
-                           "WHERE LOWER(table_name) = 'certname_facts' AND LOWER(constraint_type) = 'foreign key'"))
-        constraint   (:constraint_name result)]
-    (sql/do-commands
-     (str "ALTER TABLE certname_facts DROP CONSTRAINT " constraint)))
+  (drop-foreign-keys "certname_facts")
 
   ;; Then we replace it with a foreign key to certname_facts_metadata
   (sql/do-commands
@@ -168,7 +226,76 @@
   does equality comparisons against array columns)"
   []
   (sql/do-commands
-   "DROP INDEX idx_catalog_resources_tags"))
+    "DROP INDEX idx_catalog_resources_tags"))
+
+(defn drop-classes-and-tags
+  "Removes the `classes` and `tags` tables, as this information can be derived
+  from resources."
+  []
+  (sql/drop-table :classes)
+  (sql/drop-table :tags))
+
+(defn rename-fact-column
+  "Renames the `fact` column on `certname_facts` to `name`, for consistency."
+  []
+  (sql/do-commands
+    (if (= (sql-current-connection-database-name) "PostgreSQL")
+      "ALTER TABLE certname_facts RENAME COLUMN fact TO name"
+      "ALTER TABLE certname_facts ALTER COLUMN fact RENAME TO name")
+    "ALTER INDEX idx_certname_facts_fact RENAME TO idx_certname_facts_name"))
+
+(defn add-reports-tables
+  "Add a resource_events and reports tables."
+  []
+  (sql/create-table :reports
+    ["hash" "VARCHAR(40)" "NOT NULL" "PRIMARY KEY"]
+    ["certname" "TEXT" "REFERENCES certnames(name)" "ON DELETE CASCADE"]
+    ["puppet_version" "VARCHAR(40)" "NOT NULL"]
+    ["report_format" "SMALLINT" "NOT NULL"]
+    ["configuration_version" "VARCHAR(255)" "NOT NULL"]
+    ["start_time" "TIMESTAMP WITH TIME ZONE" "NOT NULL"]
+    ["end_time" "TIMESTAMP WITH TIME ZONE" "NOT NULL"]
+    ["receive_time" "TIMESTAMP WITH TIME ZONE" "NOT NULL"])
+
+  (sql/create-table :resource_events
+    ["report" "VARCHAR(40)" "NOT NULL" "REFERENCES reports(hash)" "ON DELETE CASCADE"]
+    ["status" "VARCHAR(40)" "NOT NULL"]
+    ["timestamp" "TIMESTAMP WITH TIME ZONE" "NOT NULL"]
+    ["resource_type" "TEXT" "NOT NULL"]
+    ["resource_title" "TEXT" "NOT NULL"]
+    ;; I wish these next two could be "NOT NULL", but for now we are
+    ;; fabricating skipped resources as events, and in those cases we don't
+    ;; have any legitimate values to put into these fields.
+    ["property" "VARCHAR(40)"]
+    ["new_value" "TEXT"]
+    ["old_value" "TEXT"]
+    ["message" "TEXT"]
+    ; we can't set the "correct" primary key because `property` is nullable
+    ; (because of skipped resources).
+    ; We decided to just use a UNIQUE constraint for now, but another option
+    ; would be to split this out into two tables.
+    ["CONSTRAINT constraint_resource_events_unique UNIQUE (report, resource_type, resource_title, property)"])
+
+  (sql/do-commands
+    "CREATE INDEX idx_reports_certname ON reports(certname)")
+
+  ; I presume we'll be doing a decent number of queries sorted by a timestamp,
+  ; and this seems like the most likely candidate out of the timestamp fields
+  (sql/do-commands
+    "CREATE INDEX idx_reports_end_time ON reports(end_time)")
+
+  (sql/do-commands
+    "CREATE INDEX idx_resource_events_report ON resource_events(report)")
+
+  (sql/do-commands
+    "CREATE INDEX idx_resource_events_resource_type ON resource_events(resource_type)")
+
+  (sql/do-commands
+    "CREATE INDEX idx_resource_events_resource_type_title ON resource_events(resource_type, resource_title)")
+
+  (sql/do-commands
+    "CREATE INDEX idx_resource_events_timestamp ON resource_events(timestamp)"))
+
 
 ;; The available migrations, as a map from migration version to migration
 ;; function.
@@ -178,23 +305,14 @@
    3 add-catalog-timestamps
    4 add-certname-facts-metadata-table
    5 add-missing-indexes
-   6 drop-old-tags-index})
+   6 drop-old-tags-index
+   7 drop-classes-and-tags
+   8 rename-fact-column
+   9 add-reports-tables})
 
 (def desired-schema-version (apply max (keys migrations)))
 
-(defn schema-version
-  "Returns the current version of the schema, or 0 if the schema
-version can't be determined."
-  []
-  (try
-    (let [query   "SELECT version FROM schema_migrations ORDER BY version DESC LIMIT 1"
-          results (sql/transaction
-                   (query-to-vec query))]
-      (:version (first results)))
-    (catch java.sql.SQLException e
-      0)))
-
-(defn- record-migration!
+(defn record-migration!
   "Records a migration by storing its version in the schema_migrations table,
 along with the time at which the migration was performed."
   [version]
@@ -203,6 +321,20 @@ along with the time at which the migration was performed."
    "INSERT INTO schema_migrations (version, time) VALUES (?, ?)"
    [version (to-timestamp (now))]))
 
+(defn applied-migrations
+  "Returns a collection of migrations that have been run against this database
+  already, ordered from oldest to latest."
+  []
+  {:post  [(sorted? %)
+           (set? %)
+           (apply < 0 %)]}
+  (try
+    (let [query   "SELECT version FROM schema_migrations ORDER BY version"
+          results (sql/transaction (query-to-vec query))]
+      (apply sorted-set (map :version results)))
+    (catch java.sql.SQLException e
+      (sorted-set))))
+
 (defn pending-migrations
   "Returns a collection of pending migrations, ordered from oldest to latest."
   []
@@ -210,24 +342,23 @@ along with the time at which the migration was performed."
           (sorted? %)
           (apply < 0 (keys %))
           (<= (count %) (count migrations))]}
-  (let [current-version (schema-version)]
-    (into (sorted-map)
-          (filter #(> (key %) current-version) migrations)) ))
+  (let [pending (difference (utils/keyset migrations) (applied-migrations))]
+      (into (sorted-map)
+        (select-keys migrations pending))))
 
 (defn migrate!
   "Migrates database to the latest schema version. Does nothing if database is
 already at the latest schema version."
   []
-  (let [current-schema-version (schema-version)]
-    (when (< desired-schema-version current-schema-version)
-      (throw (IllegalStateException.
-              (format "This version of PuppetDB works with schema version %d, but the DB is at version %d"
-                      desired-schema-version current-schema-version)))))
+  (if-let [unexpected (first (difference (applied-migrations) (utils/keyset migrations)))]
+    (throw (IllegalStateException.
+              (format "Your PuppetDB database contains a schema migration numbered %d, but this version of PuppetDB does not recognize that version."
+                    unexpected))))
 
   (if-let [pending (seq (pending-migrations))]
     (sql/transaction
      (doseq [[version migration] pending]
-       (log/info (format "Migrating to version %d" version))
+       (log/info (format "Applying migration version %d" version))
        (migration)
        (record-migration! version)))
     (log/info "There are no pending migrations")))
