@@ -8,7 +8,7 @@
 ;; database, yet not be associated with a catalog. This is done as a
 ;; performance optimization.
 ;;
-;; * edges, tags, and classes are associated with a single catalog
+;; * edges are associated with a single catalog
 ;;
 ;; * catalogs are associated with a single certname
 ;;
@@ -20,20 +20,21 @@
 
 (ns com.puppetlabs.puppetdb.scf.storage
   (:require [com.puppetlabs.puppetdb.catalog :as cat]
+            [com.puppetlabs.puppetdb.report :as report]
             [com.puppetlabs.utils :as utils]
             [com.puppetlabs.jdbc :as jdbc]
             [clojure.java.jdbc :as sql]
             [clojure.string :as str]
             [clojure.tools.logging :as log]
             [cheshire.core :as json])
-  (:use [clj-time.coerce :only [to-timestamp]]
-        [clj-time.core :only [ago days now]]
-        [clojure.core.memoize :only [memo-lru]]
+  (:use [clj-time.coerce :only [to-string to-timestamp]]
+        [clj-time.core :only [ago secs now]]
         [metrics.meters :only (meter mark!)]
         [metrics.counters :only (counter inc! value)]
         [metrics.gauges :only (gauge)]
         [metrics.histograms :only (histogram update!)]
-        [metrics.timers :only (timer time!)]))
+        [metrics.timers :only (timer time!)]
+        [com.puppetlabs.jdbc :only [query-to-vec dashes->underscores]]))
 
 (defn sql-current-connection-database-name
   "Return the database product name currently in use."
@@ -45,12 +46,23 @@
 (defn sql-current-connection-database-version
   "Return the version of the database product currently in use."
   []
+  {:post [(every? integer? %)
+          (= (count %) 2)]}
   (let [db-metadata (.. (sql/find-connection)
                       (getMetaData))
         major (.getDatabaseMajorVersion db-metadata)
         minor (.getDatabaseMinorVersion db-metadata)]
     [major minor]))
 
+(defn sql-current-connection-table-names
+  "Return all of the table names that are present in the database based on the
+  current connection.  This is most useful for debugging / testing  purposes
+  to allow introspection on the database.  (Some of our unit tests rely on this.)"
+  []
+  (let [query   "SELECT table_name FROM information_schema.tables WHERE LOWER(table_schema) = 'public'"
+        results (sql/transaction (query-to-vec query))]
+    (map :table_name results)))
+
 (defn to-jdbc-varchar-array
   "Takes the supplied collection and transforms it into a
   JDBC-appropriate VARCHAR array."
@@ -63,7 +75,7 @@
 (defmulti sql-array-type-string
   "Returns a string representing the correct way to declare an array
   of the supplied base database type."
-  ;; Dispatch based on databsae from the metadata of DB connection at the time
+  ;; Dispatch based on database from the metadata of DB connection at the time
   ;; of call; this copes gracefully with multiple connection types.
   (fn [_] (sql-current-connection-database-name)))
 
@@ -82,20 +94,30 @@ must be supplied as the value to be matched."
   number, or to NULL if it is not numeric."
   (fn [_] (sql-current-connection-database-name)))
 
+(defmulti sql-regexp-match
+  "Returns db-specific code for performing a regexp match"
+  (fn [_] (sql-current-connection-database-name)))
+
+(defmulti sql-regexp-array-match
+  "Returns db-specific code for performing a regexp match against the
+  contents of an array. If any of the array's items match the supplied
+  regexp, then that satisfies the match."
+  (fn [_ _] (sql-current-connection-database-name)))
+
 (defmethod sql-array-type-string "PostgreSQL"
   [basetype]
   (format "%s ARRAY[1]" basetype))
 
+(defmethod sql-array-type-string "HSQL Database Engine"
+  [basetype]
+  (format "%s ARRAY[%d]" basetype 65535))
+
 (defmethod sql-array-query-string "PostgreSQL"
   [column]
   (if (pos? (compare (sql-current-connection-database-version) [8 1]))
     (format "ARRAY[?::text] <@ %s" column)
     (format "? = ANY(%s)" column)))
 
-(defmethod sql-array-type-string "HSQL Database Engine"
-  [basetype]
-  (format "%s ARRAY[%d]" basetype 65535))
-
 (defmethod sql-array-query-string "HSQL Database Engine"
   [column]
   (format "? IN (UNNEST(%s))" column))
@@ -114,6 +136,30 @@ must be supplied as the value to be matched."
                "ELSE NULL END")
           column column column column))
 
+(defmethod sql-regexp-match "PostgreSQL"
+  [column]
+  (format "%s ~ ?" column))
+
+(defmethod sql-regexp-match "HSQL Database Engine"
+  [column]
+  (format "REGEXP_SUBSTRING(%s, ?) IS NOT NULL" column))
+
+(defmethod sql-regexp-array-match "PostgreSQL"
+  [table column]
+  (format "EXISTS(SELECT 1 FROM UNNEST(%s) WHERE UNNEST ~ ?)" column))
+
+(defmethod sql-regexp-array-match "HSQL Database Engine"
+  [table column]
+  ;; What evil have I wrought upon the land? Good gravy.
+  ;;
+  ;; This is entirely due to the fact that HSQLDB doesn't support the
+  ;; UNNEST operator referencing a column from an outer table. UNNEST
+  ;; *has* to come after the parent table in the FROM clause of a
+  ;; separate SQL statement.
+  (format (str "EXISTS(SELECT 1 FROM %s %s_copy, UNNEST(%s) AS T(the_tag) "
+               "WHERE %s.%s=%s_copy.%s AND REGEXP_SUBSTRING(the_tag, ?) IS NOT NULL)")
+          table table column table column table column))
+
 (def ns-str (str *ns*))
 
 ;; ## Performance metrics
@@ -125,11 +171,6 @@ must be supplied as the value to be matched."
 ;;
 ;; * `:add-catalog`: the time it takes to persist a catalog
 ;;
-;; * `:add-classes`: the time it takes to persist just a catalog's
-;;   classes
-;;
-;; * `:add-tags`: the time it takes to persist just a catalog's tags
-;;
 ;; * `:add-resources`: the time it takes to persist just a catalog's
 ;;   resources
 ;;
@@ -165,8 +206,6 @@ must be supplied as the value to be matched."
 ;;
 (def metrics
   {
-   :add-classes       (timer [ns-str "default" "add-classes"])
-   :add-tags          (timer [ns-str "default" "add-tags"])
    :add-resources     (timer [ns-str "default" "add-resources"])
    :add-edges         (timer [ns-str "default" "add-edges"])
 
@@ -187,6 +226,8 @@ must be supplied as the value to be matched."
                                (float (utils/quotient dupes (+ dupes new)))))
 
    :replace-facts     (timer [ns-str "default" "replace-facts-time"])
+
+   :store-report      (timer [ns-str "default" "store-report-time"])
    })
 
 (defn db-serialize
@@ -253,18 +294,25 @@ must be supplied as the value to be matched."
     (:deactivated (first result-set))))
 
 (defn activate-node!
-  "Reactivate the given host"
+  "Reactivate the given host.  Adds the host to the database if it was not
+  already present."
   [certname]
   {:pre [(string? certname)]}
+  (when-not (certname-exists? certname)
+    (add-certname! certname))
   (sql/update-values :certnames
                      ["name=?" certname]
                      {:deactivated nil}))
 
 (defn maybe-activate-node!
   "Reactivate the given host, only if it was deactivated before `time`.
-  Returns true if the node is activated, or if it was already active."
+  Returns true if the node is activated, or if it was already active.
+
+  Adds the host to the database if it was not already present."
   [certname time]
   {:pre [(string? certname)]}
+  (when-not (certname-exists? certname)
+    (add-certname! certname))
   (let [timestamp (to-timestamp time)
         replaced  (sql/update-values :certnames
                                      ["name=? AND (deactivated<? OR deactivated IS NULL)" certname timestamp]
@@ -300,24 +348,6 @@ must be supplied as the value to be matched."
     ["SELECT 1 FROM catalogs WHERE hash=? LIMIT 1" hash]
     (pos? (count result-set))))
 
-(defn add-classes!
-  "Given a catalog-hash and a list of classes, persist them in the db"
-  [catalog-hash classes]
-  {:pre [(string? catalog-hash)
-         (coll? classes)]}
-  (let [default-row {:catalog catalog-hash}
-        classes     (map #(assoc default-row :name %) classes)]
-    (apply sql/insert-records :classes classes)))
-
-(defn add-tags!
-  "Given a catalog-hash and a list of tags, persist them in the db"
-  [catalog-hash tags]
-  {:pre [(string? catalog-hash)
-         (coll? tags)]}
-  (let [default-row {:catalog catalog-hash}
-        tags        (map #(assoc default-row :name %) tags)]
-    (apply sql/insert-records :tags tags)))
-
 (defn resources-exist?
   "Given a collection of resource-hashes, return the subset that
   already exist in the database."
@@ -362,7 +392,7 @@ must be supplied as the value to be matched."
 
 ;; Size of the cache is based on the number of unique resources in a
 ;; "medium" site persona
-(def resource-identity-hash* (memo-lru resource-identity-hash* 40000))
+(def resource-identity-hash* (utils/bounded-memoize resource-identity-hash* 40000))
 
 (defn resource-identity-hash
   "Compute a hash for a given resource that will uniquely identify it
@@ -382,10 +412,10 @@ must be supplied as the value to be matched."
   catalog. Unlike `resource-identity-hash`, this string will also
   include the resource metadata. This function is used as part of
   determining whether a catalog needs to be stored."
-  [{:keys [type title parameters tags exported file line] :as resource}]
+  [{:keys [type title parameters exported file line] :as resource}]
   {:pre  [(map? resource)]
    :post [(string? %)]}
-  (pr-str [type title (sort tags) exported file line (sort parameters)]))
+  (pr-str [type title exported file line (sort parameters)]))
 
 (defn- resource->values
   "Given a catalog-hash, a resource, and a truthy value indicating
@@ -477,14 +507,14 @@ must be supplied as the value to be matched."
 
   This hash is useful for situations where you'd like to determine
   whether or not two catalogs contain the same things (edges,
-  resources, tags, classes, etc).
+  resources, etc).
 
   Note that this hash *cannot* be used to uniquely identify a catalog
   within a population! This is because we're only examing a subset of
   a catalog's attributes. For example, two otherwise identical
   catalogs with different :version's would have the same similarity
   hash, but don't represent the same catalog across time."
-  [{:keys [certname classes tags resources edges] :as catalog}]
+  [{:keys [certname resources edges] :as catalog}]
   ;; deepak: This could probably be coded more compactly by just
   ;; dissociating the keys we don't want involved in the computation,
   ;; but I figure that for safety's sake, it's better to be very
@@ -492,8 +522,6 @@ must be supplied as the value to be matched."
   ;; about when we think about "uniqueness".
   (-> (sorted-map)
       (assoc :certname certname)
-      (assoc :classes (sort classes))
-      (assoc :tags (sort tags))
       (assoc :resources (sort (for [[ref resource] resources]
                                 (catalog-resource-identity-string resource))))
       (assoc :edges (sort (map edge-identity-string edges)))
@@ -503,9 +531,9 @@ must be supplied as the value to be matched."
 (defn add-catalog!
   "Persist the supplied catalog in the database, returning its
   similarity hash"
-  [{:keys [api-version version resources classes edges tags] :as catalog}]
+  [{:keys [api-version version resources edges] :as catalog}]
   {:pre [(number? api-version)
-         (every? coll? [classes tags edges])
+         (coll? edges)
          (map? resources)]}
 
   (time! (:add-catalog metrics)
@@ -525,10 +553,6 @@ must be supplied as the value to be matched."
               (when-not exists?
                 (inc! (:new-catalog metrics))
                 (add-catalog-metadata! hash api-version version)
-                (time! (:add-classes metrics)
-                       (add-classes! hash classes))
-                (time! (:add-tags metrics)
-                       (add-tags! hash tags))
                 (let [refs-to-hashes (zipmap (keys resources) resource-hashes)]
                   (time! (:add-resources metrics)
                          (add-resources! hash resources refs-to-hashes))
@@ -563,7 +587,7 @@ must be supplied as the value to be matched."
   [certname]
   (sql/with-query-results result-set
     ["SELECT catalog FROM certname_catalogs WHERE certname=?" certname]
-    (vec (map :catalog result-set))))
+    (mapv :catalog result-set)))
 
 (defn catalog-newer-than?
   "Returns true if the most current catalog for `certname` is more recent than
@@ -624,12 +648,12 @@ must be supplied as the value to be matched."
 
 (defn add-facts!
   "Given a certname and a map of fact names to values, store records for those
-facts associated with the certname."
+  facts associated with the certname."
   [certname facts timestamp]
   {:pre [(utils/datetime? timestamp)]}
   (let [default-row {:certname certname}
         rows        (for [[fact value] facts]
-                      (assoc default-row :fact fact :value value))]
+                      (assoc default-row :name fact :value value))]
     (sql/insert-record :certname_facts_metadata
                        {:certname certname :timestamp (to-timestamp timestamp)})
     (apply sql/insert-records :certname_facts rows)))
@@ -649,3 +673,108 @@ facts associated with the certname."
          (sql/transaction
           (delete-facts! name)
           (add-facts! name values timestamp))))
+
+
+(defn resource-event-identity-string
+  "Compute a hash for a resource event
+
+  This hash is useful for situations where you'd like to determine
+  whether or not two resource events are identical (resource type, resource title,
+  property, values, status, timestamp, etc.)
+  "
+  [{:keys [resource-type resource-title property timestamp status old-value
+           new-value message] :as resource-event}]
+  (-> (sort { :resource-type resource-type
+              :resource-title resource-title
+              :property property
+              :timestamp timestamp
+              :status status
+              :old-value old-value
+              :new-value new-value
+              :message message})
+      (pr-str)
+      (utils/utf8-string->sha1)))
+
+(defn report-identity-string
+  "Compute a hash for a report's content
+
+  This hash is useful for situations where you'd like to determine
+  whether or not two reports contain the same things (certname,
+  configuration version, timestamps, events).
+  "
+  [{:keys [certname puppet-version report-format configuration-version
+           start-time end-time resource-events] :as report}]
+  (-> (sorted-map)
+    (assoc :certname certname)
+    (assoc :puppet-version puppet-version)
+    (assoc :report-format report-format)
+    (assoc :configuration-version configuration-version)
+    (assoc :start-time start-time)
+    (assoc :end-time end-time)
+    (assoc :resource-events (sort (map resource-event-identity-string resource-events)))
+    (pr-str)
+    (utils/utf8-string->sha1)))
+
+(defn add-report!
+  "Add a report and all of the associated events to the database."
+  [{:keys [puppet-version certname report-format configuration-version
+           start-time end-time resource-events]
+    :as report}
+   timestamp]
+  {:pre [(map? report)
+         (utils/datetime? timestamp)]}
+  (let [report-hash         (report-identity-string report)
+        resource-event-rows (map #(-> %
+                                     (update-in [:timestamp] to-timestamp)
+                                     (update-in [:old-value] db-serialize)
+                                     (update-in [:new-value] db-serialize)
+                                     (assoc :report report-hash)
+                                     ((partial utils/mapkeys dashes->underscores)))
+                                  resource-events)]
+    (time! (:store-report metrics)
+      (sql/transaction
+        ;; TODO: should probably do some checking / error-handling around
+        ;; whether or not the report id already exists
+        (sql/insert-record :reports
+          { :hash                   report-hash
+            :puppet_version         puppet-version
+            :certname               certname
+            :report_format          report-format
+            :configuration_version  configuration-version
+            :start_time             (to-timestamp start-time)
+            :end_time               (to-timestamp end-time)
+            :receive_time           (to-timestamp timestamp)})
+        (apply sql/insert-records :resource_events resource-event-rows)))))
+
+(defn delete-reports-older-than!
+  "Delete all reports in the database which have an `end-time` that is prior to
+  the specified date/time."
+  [time]
+  {:pre [(utils/datetime? time)]}
+  (sql/delete-rows :reports ["end_time < ?" (to-timestamp time)]))
+
+(defmulti db-deprecated?
+  "Returns a vector with a boolean indicating if database type and version is
+  marked for deprecation. The second element in the vector is a string
+  explaining the deprecation."
+  (fn [dbtype version] dbtype))
+
+(defmethod db-deprecated? "PostgreSQL"
+  [_ version]
+  (if (pos? (compare [8 4] version))
+    [true "PostgreSQL DB 8.3 and older are deprecated and won't be supported in the future."]
+    [false nil]))
+
+(defmethod db-deprecated? :default
+  [_ _]
+  [false nil])
+
+(defn warn-on-db-deprecation!
+  "Get metadata about the current connection and warn if the database we are
+  using is deprecated."
+  []
+  (let [version    (sql-current-connection-database-version)
+        dbtype     (sql-current-connection-database-name)
+        [deprecated? message] (db-deprecated? dbtype version)]
+    (when deprecated?
+      (log/warn message))))
