@@ -23,10 +23,27 @@
         [clojure.set :only (difference union)]
         [clojure.stacktrace :only (print-cause-trace)]
         [clj-time.core :only [now]]
-        [clj-time.coerce :only [ICoerce]]
+        [clj-time.coerce :only [ICoerce to-date-time]]
         [clj-time.format :only [formatters unparse]]
         [slingshot.slingshot :only (try+ throw+)]))
 
+;; ## Type checking
+
+(defn array?
+  "Returns true if `x` is an array"
+  [x]
+  (-?> x
+    (class)
+    (.isArray)))
+
+(defn datetime?
+  "Predicate returning whether or not the supplied object is
+  convertible to a Joda DateTime"
+  [x]
+  (and
+    (satisfies? ICoerce x)
+    (to-date-time x)))
+
 ;; ## I/O
 
 (defn lines
@@ -79,6 +96,7 @@
    :post [(or (number? %) (nil? %))]}
   ((some-fn parse-int parse-float) s))
 
+
 ;; ## Collection operations
 
 (defn symmetric-difference
@@ -99,10 +117,18 @@
        (constructor item))))
 
 (defn mapvals
-  "Return map `m`, with each value transformed by function `f`"
-  [f m]
-  (into {} (concat (for [[k v] m]
-                     [k (f v)]))))
+  "Return map `m`, with each value transformed by function `f`.
+
+  You may also provide an optional list of keys `ks`; if provided, only the
+  specified keys will be modified."
+  ([f m]
+    (into {} (for [[k v] m] [k (f v)])))
+  ([f ks m]
+    ;; would prefer to share code between the two implementations here, but
+    ;; the `into` is much faster for the base case and the reduce is much
+    ;; faster for any case where we're operating on a subset of the keys.
+    ;; It seems like `select-keys` is fairly expensive.
+    (reduce (fn [m k] (update-in m [k] f)) m ks)))
 
 (defn mapkeys
   "Return map `m`, with each key transformed by function `f`"
@@ -110,12 +136,19 @@
   (into {} (concat (for [[k v] m]
                      [(f k) v]))))
 
-(defn array?
-  "Returns true if `x` is an array"
-  [x]
-  (-?> x
-       (class)
-       (.isArray)))
+(defn maptrans
+  "Return map `m`, with values transformed according to the key-to-function
+  mappings specified in `keys-fns`.  `keys-fns` should be a map whose keys
+  are lists of keys from `m`, and whose values are functions to apply to those
+  keys.
+
+  Example: `(maptrans {[:a, :b] inc [:c] dec} {:a 1 :b 1 :c 1})` yields `{:a 2, :c 0, :b 2}`"
+  [keys-fns m]
+  {:pre [(map? keys-fns)
+         (every? (fn [[ks fn]] (and (coll? ks) (ifn? fn))) keys-fns)
+         (map? m)]}
+  (let [ks (keys keys-fns)]
+    (reduce (fn [m k] (mapvals (keys-fns k) k m)) m ks)))
 
 (defn keyset
   "Retuns the set of keys from the supplied map"
@@ -133,12 +166,6 @@
 
 ;; ## Date and Time
 
-(defn datetime?
-  "Predicate returning whether or not the supplied object is
-  convertible to a Joda DateTime"
-  [x]
-  (satisfies? ICoerce x))
-
 (defn timestamp
   "Returns a timestamp string for the given `time`, or the current time if none
   is provided. The format of the timestamp is eg. 2012-02-23T22:01:39.539Z."
@@ -468,6 +495,22 @@
     (let [bytes (.getBytes s "UTF-8")]
       (digest-func "sha-1" [bytes]))))
 
+(defn bounded-memoize
+  "Similar to memoize, but the cache will be reset if the number of entries
+  exceeds the specified `bound`."
+  [f bound]
+  {:pre [(integer? bound)
+         (pos? bound)]}
+  (let [cache (atom {})]
+    (fn [& args]
+      (if-let [e (find @cache args)]
+        (val e)
+        (let [v (apply f args)]
+          (when (> (count @cache) bound)
+            (reset! cache {}))
+          (swap! cache assoc args v)
+          v)))))
+
 ;; ## UUID handling
 
 (defn uuid
